#include <libtcc.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

void handle_error(void *opaque, const char *msg)
{
    fprintf(opaque, "%s\n", msg);
}

typedef char * (*codefn)();

TCCState * init_tcc() {
    TCCState *s;

    s = tcc_new();
    if (!s) {
		printf("Tcc init err\n");
        exit(1);
    }

    tcc_set_error_func(s, stderr, handle_error);
    tcc_set_output_type(s, TCC_OUTPUT_MEMORY);
	return s;
}


int main(int argc, char **argv)
{
	if (argc <= 1) {
		printf("cpointerc: missing argument\n");
		return 1;
	}

	FILE * file = fopen(argv[1], "r");
	if (file == NULL) {
		printf("%s:%s\n", argv[1], strerror(errno));
		return 1;
	}


	fseek(file, 0, SEEK_END);

	long fsize = ftell(file);

	rewind(file);

	char * layer1 = calloc(sizeof(char), fsize);

	fread(layer1, sizeof(char), fsize, file);

	fclose(file);

    int i;
	
	TCCState * s = init_tcc();

    if (tcc_compile_string(s, layer1) == -1)
        return 1;

    /* tcc_add_symbol(s, "add", add); */
    /* tcc_add_symbol(s, "hello", hello); */
    if (tcc_relocate(s) < 0)
        return 1;

    /* get entry symbol */
    codefn layer1_entry = tcc_get_symbol(s, "entry");
    
    if (!layer1_entry)
        return 1;

	//printf("calling stage 1\n");
    char * mem = layer1_entry();

	//printf("done stage 1\n");
	tcc_delete(s);
	s = init_tcc();

	
	//printf("compiling: \n%s\n", mem);
	if(tcc_compile_string(s, mem) == -1) {
		return 1;
	}

	if (tcc_relocate(s) < 0) {
		return 1;
	}
	int (*newmn)() = tcc_get_symbol(s, "newmain");
	if (newmn == NULL) {
		printf("newmain err\n");
		return 1;
	}
	newmn();
    /* delete the state */
    tcc_delete(s);

    return 0;
}
